/* meArm library York Hack Space May 2014
 * A simple control library for Phenoptix' meArm
 * Usage:
 *   meArm arm;
 *   arm.begin(1, 10, 9, 6);
 *   arm.openGripper();
 *   arm.gotoPoint(-80, 100, 140);
 *   arm.closeGripper();
 *   arm.gotoPoint(70, 200, 10);
 *   arm.openGripper();
 */
#ifndef QGPARMBOT_H
#define QGPARMBOT_H

#include <EEPROM.h>
#include <Arduino.h>
#include <Servo.h>

#define ARM_MODE_IDLE 0
#define ARM_MODE_RECODE 1
#define ARM_MODE_REPLAY 2

#define AXIS_X 1
#define AXIS_Y 2
#define AXIS_Z 3
#define AXIS_G 4

const float pi = 3.14159265359;
const int PT_LEN = sizeof(float) * 4;

extern int QGP_armMode;
struct ServoInfo
{
  int n_min, n_max; // PWM 'soft' limits - should be just within range
  float gain;       // PWM per radian
  float zero;       // Theoretical PWM for zero angle
};

struct PT
{
  float x;
  float y;
  float z;
  float grp;
};

class QGPArmbot
{
public:
  //Full constructor uses calibration data, or can just give pins
  QGPArmbot(int sweepMinBase = 145, int sweepMaxBase = 49, float angleMinBase = -pi / 4, float angleMaxBase = pi / 4,
            int sweepMinShoulder = 118, int sweepMaxShoulder = 22, float angleMinShoulder = pi / 4, float angleMaxShoulder = 3 * pi / 4,
            int sweepMinElbow = 144, int sweepMaxElbow = 36, float angleMinElbow = pi / 4, float angleMaxElbow = -pi / 4,
            int sweepMinGripper = 0, int sweepMaxGripper = 90, float angleMinGripper = 0, float angleMaxGripper = pi / 2);
  //required before running
  void begin(int pinBase = 9, int pinElbow = 8, int pinShoulder = 7, int pinGripper = 6);
  void end();
  //Travel smoothly from current point to another point
  void gotoPoint(float x, float y, float z);
  //Set servos to reach a certain point directly without caring how we get there
  void goDirectlyTo(float x, float y, float z);

  //Same as above but for cylindrical polar coodrinates
  void gotoPointCylinder(float theta, float r, float z);
  void goDirectlyToCylinder(float theta, float r, float z);

  //Grab something
  void openGripper(int gp = 0);
  //Let go of something
  void closeGripper(int gp = 90);
  //Check to see if possible
  bool isReachable(float x, float y, float z);
  //Current x, y and z
  float getX();
  float getY();
  float getZ();

  int getArmMode();
  void setArmMode(int _mode);

  void moveXYZG(int xzy, int n);

  float getR();
  float getTheta();
  void record();
  void replay(int _loop = -1, int _delay = 1000);
  void cleanRecord();

private:
  void polarToCartesian(float theta, float r, float &x, float &y);
  float _x, _y, _z, _grp, _eex, _eey, _eez, _eegrp;
  float _r, _t;
  Servo _base, _shoulder, _elbow, _gripper;
  ServoInfo _svoBase, _svoShoulder, _svoElbow, _svoGripper;
  int _pinBase, _pinShoulder, _pinElbow, _pinGripper;
  byte eeAddress;
};

#endif
